﻿/*-------------------------------------------------------------------------
 Cipher algorithm interface
 
 File name   : Cipher.cs
 Version     : Version 1.0
 Created     : Aug/20/2008
 Last update : Sep/03/2008
 Desgined by : Takeshi Sugawara
 Modified by : Toshihiro Katashita
 
 
 Copyright (C) 2008 AIST and Tohoku Univ.
 
 By using this code, you agree to the following terms and conditions.
 
 This code is copyrighted by AIST and Tohoku University ("us").
 
 Permission is hereby granted to copy, reproduce, redistribute or
 otherwise use this code as long as: there is no monetary profit gained
 specifically from the use or reproduction of this code, it is not sold,
 rented, traded or otherwise marketed, and this copyright notice is
 included prominently in any copy made.
 
 We shall not be liable for any damages, including without limitation
 direct, indirect, incidental, special or consequential damages arising
 from the use of this code.
 
 When you publish any results arising from the use of this code, we will
 appreciate it if you can cite our webpage
 (http://staff.aist.go.jp/akashi.satoh/SASEBO/en/).
 -------------------------------------------------------------------------*/

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace CipherTool
{
	//================================================================ CipherInterface
	public interface IBlockCipher
	{
		void setKey (byte[] key);
		byte[] encrypt (byte[] pt);
		byte[] decrypt (byte[] ct);
	}

	//================================================================ Utility
	public class Utility
	{
		public static UInt16 ByteArrayToUInt16(Byte[] ByteArray)
		{
			UInt16 x = 0;
			x += (UInt16)(((UInt16)ByteArray[0]) << 8);
			x += (UInt16)(((UInt16)ByteArray[1]));
			return x;
		}

		public static Byte[] UInt32ToByteArray(UInt32 x)
		{
			Byte[] ByteArray = new Byte[4];
			ByteArray[0] = (Byte)(x >> 24);
			ByteArray[1] = (Byte)(x >> 16);
			ByteArray[2] = (Byte)(x >> 8);
			ByteArray[3] = (Byte)(x);
			return ByteArray;
		}

		public static UInt32 ByteArrayToUInt32(Byte[] ByteArray)
		{
			UInt32 x = 0;
			x += ((UInt32)ByteArray[0]) << 24;
			x += ((UInt32)ByteArray[1]) << 16;
			x += ((UInt32)ByteArray[2]) << 8;
			x += ((UInt32)ByteArray[3]);
			return x;
		}

		public static string UInt32ArrayToString(UInt32[] UInt32Array)
		{
			string tmpStr = "";
			foreach (UInt32 i in UInt32Array)
			{
				tmpStr += string.Format("{0:x8} ", i);
			}
			return tmpStr.Trim();
		}

		public static string UInt16ArrayToString(UInt16[] UInt16Array)
		{
			string tmpStr = "";
			foreach (UInt16 i in UInt16Array)
			{
				tmpStr += string.Format("{0:x4} ", i);
			}
			return tmpStr.Trim();
		}

		public static string ByteArrayToString(Byte[] ByteArray)
		{
			string tmpStr = "";
			foreach (Byte i in ByteArray)
			{
				tmpStr += string.Format("{0:x2} ", i);
			}
			return tmpStr.Trim();
		}

		public static string ByteArrayToCSV(Byte[] ByteArray)
		{
			string tmpStr = "";
			foreach (Byte i in ByteArray)
			{
				tmpStr += string.Format("{0}, ", i);
			}
			return tmpStr.Substring(0, tmpStr.Length - 2);
		}

		public static void UInt32ToBytes(UInt32 ABCD, ref Byte A, ref Byte B, ref Byte C, ref Byte D)
		{
			Byte[] ByteArray = new Byte[4];
			ByteArray = Utility.UInt32ToByteArray(ABCD);
      A = ByteArray[0]; B = ByteArray[1]; C = ByteArray[2]; D = ByteArray[3];
		}

		public static Byte[] UInt32ArrayToByteArray(UInt32[] UInt32Array)
		{
			Byte[] ByteArray = new Byte[UInt32Array.Length * 4];
			for (int i = 0; i < UInt32Array.Length; ++i)
			{
				ByteArray[4 * i + 0] = (Byte)(UInt32Array[i] >> 24);
				ByteArray[4 * i + 1] = (Byte)(UInt32Array[i] >> 16);
				ByteArray[4 * i + 2] = (Byte)(UInt32Array[i] >> 8);
				ByteArray[4 * i + 3] = (Byte)(UInt32Array[i]);
			}
			return ByteArray;
		}

		public static UInt32[] ByteArrayToUInt32Array(Byte[] ByteArray)
		{
			if (ByteArray.Length % 4 != 0)
				throw new SystemException("Invalid size of an argument array");

			int length = ByteArray.Length / 4;
			UInt32[] UInt32Array = new UInt32[length];
			for (int i = 0; i < length; ++i)
			{
				UInt32Array[i] = (((UInt32)ByteArray[4 * i]    ) << 24)
											 | (((UInt32)ByteArray[4 * i + 1]) << 16)
											 | (((UInt32)ByteArray[4 * i + 2]) << 8)
											 | ( (UInt32)ByteArray[4 * i + 3]);
			}
			return UInt32Array;
		}

		public static UInt32 RotateLeft(UInt32 Data, int Amount)
		{
			if (Amount > 32)
				throw new SystemException("Invalid argument: Amound");

			UInt32 temp = 0;
			temp |= Data << Amount;
			temp |= Data >> (32 - Amount);
			return temp;
		}

		public static UInt32 RotateRight(UInt32 Data, int Amount)
		{
			if (Amount > 32)
				throw new SystemException("Invalid argument: Amound");

			UInt32 temp = 0;
			temp |= Data >> Amount;
			temp |= Data << (32 - Amount);
			return temp;
		}

		public static UInt32[] StringToUInt32Array(string Str, int Size)
		{
			UInt32[] UInt32Arary = new UInt32[Size];
			string SubStr;

			if (Str.Length != 8 * Size)
				throw new System.Exception("Invalid length of argument string in StringToUInt32Array");
			else
			{
				for (int i = 0; i < Size; ++i)
				{
					SubStr = Str.Substring(i * 8, 8);
					UInt32Arary[i] = Convert.ToUInt32(SubStr, 16);
				}
				return UInt32Arary;
			}
		}

		public static bool CompareUInt32Array(UInt32[] x, UInt32[] y)
		{
			for (int i = 0; i < x.Length; ++i)
			{
				if (x[i] != y[i]) return false;
			}
			return true;
		}
	}
}
